/*
 * Copyright (c) 2006-2011 LodgON  (http://www.lodgon.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.lodgon.dali.core.oauth.util;

import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;

/**
 * A helper class that helps with generating random Strings and hashes.
 *
 * @author joeri
 */
public class StringUtil {

	/**
	 * Converts the byte array into a hexadecimal String. Each byte from the array
	 * is converted to its hexadecimal format, which is in the range 0x0 to 0xFF.
	 * When the hexadecimal format is lower than 0x10, it will be prepended with
	 * a zero.
	 * @param input the byte array to convert.
	 * @return A hexadecimal String.
	 */
	public static String toHexString(byte[] input) {
		StringBuilder hexString = new StringBuilder();
		for (int i = 0; i < input.length; i++) {
			String hex = Integer.toHexString(0xFF & input[i]);
			if (hex.length() == 1) {
				hexString.append("0");
			}
			hexString.append(hex);
		}
		return hexString.toString();
	}

	/**
	 * Generate a String containing random characters. The characters are randomly
	 * chosen using the SecureRandom class with the algorithm SHA1PRNG.
	 *
	 * @param length the length of the random String returned. It must be greater
	 *        than 0 and it should be divisable by two.
	 * @return A String containing random characters or null if the SHA1PRNG
	 *         algorithm appears to be unavailable on the platform. It also
	 *         returns 0 when length is less than or equal to 0 or not divisable
	 *         by two.
	 */
	public static String getSecureRandomString(int length) {
		if (length <= 0 || length % 2 == 1) {
			return null;
		}

		try {
			SecureRandom sr = SecureRandom.getInstance("SHA1PRNG");

			byte[] bytes = new byte[length / 2];
			sr.nextBytes(bytes);
			return StringUtil.toHexString(bytes);
		} catch (NoSuchAlgorithmException ex) {
			return null;
		}
	}

}
